/*
 * usart_dma_test.c
 *
 * Created: 4/13/2012 10:45:32 AM
 *  Author: Doug
 */ 

/******************************************************************************
 *                      Task 3: USART using DMAC
 *
 * This task will show how to set up the USART with the DMA controller. 
 * SetupTransmitChannel and SetupReceiveChannel are using the dma_driver.c
 * The driver provides a fast setup of the DMAC. SetupTransmitChannel 
 * is setting up the Tx_Buf buffer to feed the USART data transfer register. 
 * At the receiver, another DMAC channel will receive the bytes and 
 * start filling the Rx_Buf buffer.
 *****************************************************************************/

#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include "dma_driver.h"
#include "usart_driver.h"

// The board.h header file defines which IO ports peripherals like
// Switches and LEDs are connected to.
#include "board.h"

//Default BAUD is 9600
#define BSEL_DEFAULT BSEL_9600
#define BSCALE_DEFAULT BSCALE_9600	

#define BSEL_HI_SPEED BSEL_115200
#define BSCALE_HI_SPEED BSCALE_115200

// 32MHz internal RC oscillator, calibrated to 33868800 MHz
#define BSEL_9600     878
#define BSCALE_9600   0b1110 //-2, 0b0010
#define BSEL_19200    437
#define BSCALE_19200  0b1110 //-2
#define BSEL_38400	  433
#define BSCALE_38400  0b1101 //-3, 0b0011
#define BSEL_57600    286
#define BSCALE_57600  0b1101 //-3
#define BSEL_115200   139
#define BSCALE_115200 0b1101 //-3
#define BSEL_230400   131
#define BSCALE_230400 0b1100 //-4, 0b0100
#define BSEL_460800   115
#define BSCALE_460800 0b1011 //-5, 0b0101
#define BSEL_921600   83
#define BSCALE_921600 0b1010 //-6, 0b0110

#define DMA_TX_Channel &DMA.CH0
#define DMA_RX_Channel &DMA.CH1

// USART data struct used in task
USART_data_t USART_data;

void SetupWiflyUsartInterrupts9600( void )
{
	// (TX) as output
    USART_WIFLY_PORT.DIRSET   = PIN3_bm;
    // (RX) as input
    USART_WIFLY_PORT.DIRCLR   = PIN2_bm;
    
    // Use USARTx0 and initialize buffers
    USART_InterruptDriver_Initialize(&USART_data, &USART_WIFLY, USART_DREINTLVL_LO_gc);
    
    // USARTx0, 8 Data bits, No Parity, 1 Stop bit
    USART_Format_Set(USART_data.usart, USART_CHSIZE_8BIT_gc,
                     USART_PMODE_DISABLED_gc, false);
	
	// Enable RXC interrupt
	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_LO_gc);
		 
	// Set baud rate
	USART_Baudrate_Set(&USART_WIFLY, BSEL_9600, BSCALE_9600);
    
	/* Enable both RX and TX. */
	USART_Rx_Enable(USART_data.usart);
	USART_Tx_Enable(USART_data.usart);

	// Enable PMIC interrupt level low
	PMIC.CTRL |= PMIC_LOLVLEX_bm;

	// Enable global interrupts
	sei();
}

void SetupWiflyUsartInterrupts115200( void )
{
    cli();
	
	USART_Rx_Disable(USART_data.usart);
	USART_Tx_Disable(USART_data.usart);
	
	// (TX) as output
    USART_WIFLY_PORT.DIRSET   = PIN3_bm;
    // (RX) as input
    USART_WIFLY_PORT.DIRCLR   = PIN2_bm;
    
    // Use USARTx0 and initialize buffers
    USART_InterruptDriver_Initialize(&USART_data, &USART_WIFLY, USART_DREINTLVL_LO_gc);
    
    // USARTx0, 8 Data bits, No Parity, 1 Stop bit
    USART_Format_Set(USART_data.usart, USART_CHSIZE_8BIT_gc,
                     USART_PMODE_DISABLED_gc, false);
	
	// Enable RXC interrupt
	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_LO_gc);
		 
	// Set baud rate 
    USART_Baudrate_Set(&USART_WIFLY, BSEL_115200, BSCALE_115200);
	//USART_WIFLY.BAUDCTRLA = ((uint8_t)BSEL_HI_SPEED);
	//USART_WIFLY.BAUDCTRLB = ((BSEL_HI_SPEED >> 8)) | ((BSCALE_HI_SPEED << 4));
    
	/* Enable both RX and TX. */
	USART_Rx_Enable(USART_data.usart);
	USART_Tx_Enable(USART_data.usart);

	// Enable PMIC interrupt level low
	PMIC.CTRL |= PMIC_LOLVLEX_bm;
}

void SetupWiflyUsartInterrupts230400( void )
{
    cli();
	
	USART_Rx_Disable(USART_data.usart);
	USART_Tx_Disable(USART_data.usart);
	
	// (TX) as output
    USART_WIFLY_PORT.DIRSET   = PIN3_bm;
    // (RX) as input
    USART_WIFLY_PORT.DIRCLR   = PIN2_bm;
    
    // Use USARTx0 and initialize buffers
    USART_InterruptDriver_Initialize(&USART_data, &USART_WIFLY, USART_DREINTLVL_LO_gc);
    
    // USARTx0, 8 Data bits, No Parity, 1 Stop bit
    USART_Format_Set(USART_data.usart, USART_CHSIZE_8BIT_gc,
                     USART_PMODE_DISABLED_gc, false);
	
	// Enable RXC interrupt
	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_LO_gc);
		 
	// Set baud rate 
    USART_Baudrate_Set(&USART_WIFLY, BSEL_230400, BSCALE_230400);
	//USART_WIFLY.BAUDCTRLA = ((uint8_t)BSEL_HI_SPEED);
	//USART_WIFLY.BAUDCTRLB = ((BSEL_HI_SPEED >> 8)) | ((BSCALE_HI_SPEED << 4));
    
	/* Enable both RX and TX. */
	USART_Rx_Enable(USART_data.usart);
	USART_Tx_Enable(USART_data.usart);

	// Enable PMIC interrupt level low
	PMIC.CTRL |= PMIC_LOLVLEX_bm;
}

void SetupWiflyUsartInterrupts460800 ( void )
{
    cli();
	
	USART_Rx_Disable(USART_data.usart);
	USART_Tx_Disable(USART_data.usart);
	
	// (TX) as output
    USART_WIFLY_PORT.DIRSET   = PIN3_bm;
    // (RX) as input
    USART_WIFLY_PORT.DIRCLR   = PIN2_bm;
    
    // Use USARTx0 and initialize buffers
    USART_InterruptDriver_Initialize(&USART_data, &USART_WIFLY, USART_DREINTLVL_LO_gc);
    
    // USARTx0, 8 Data bits, No Parity, 1 Stop bit
    USART_Format_Set(USART_data.usart, USART_CHSIZE_8BIT_gc,
                     USART_PMODE_DISABLED_gc, false);
	
	// Enable RXC interrupt
	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_LO_gc);
		 
	// Set baud rate 
    USART_Baudrate_Set(&USART_WIFLY, BSEL_460800 , BSCALE_460800 );
    
	/* Enable both RX and TX. */
	USART_Rx_Enable(USART_data.usart);
	USART_Tx_Enable(USART_data.usart);

	// Enable PMIC interrupt level low
	PMIC.CTRL |= PMIC_LOLVLEX_bm;
}

//use when MCU baud rate is UPDATED to 921600
void SetupWiflyUsartInterrupts921600(void)
{
	cli();
    
	_delay_ms(500);
	 USART_Baudrate_Set(&USART_WIFLY, BSEL_921600, BSCALE_921600);
	_delay_ms(500);
	
	sei();
}

//Use when MCU needs to start at this baud rate
void SetupUsartInterrupts921600(void)
{
	cli();
	
	// (TX) as output
    USART_WIFLY_PORT.DIRSET   = PIN3_bm;
    // (RX) as input
    USART_WIFLY_PORT.DIRCLR   = PIN2_bm;
    
    // Use USARTx0 and initialize buffers
    USART_InterruptDriver_Initialize(&USART_data, &USART_WIFLY, USART_DREINTLVL_LO_gc);
    
    // USARTx0, 8 Data bits, No Parity, 1 Stop bit
    USART_Format_Set(USART_data.usart, USART_CHSIZE_8BIT_gc,
                     USART_PMODE_DISABLED_gc, false);
	
	// Enable RXC interrupt
	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_LO_gc);
		 
	// Set baud rate 
	_delay_ms(500);
    USART_Baudrate_Set(&USART_WIFLY, BSEL_921600, BSCALE_921600);
	_delay_ms(500);
    
	/* Enable both RX and TX. */
	USART_Rx_Enable(USART_data.usart);
	USART_Tx_Enable(USART_data.usart);

	// Enable PMIC interrupt level low
	PMIC.CTRL |= PMIC_LOLVLEX_bm;
	
	sei();
}

void DisableWiflyUsartRxISR(void)
{

	USART_RxdInterruptLevel_Set(USART_data.usart, USART_RXCINTLVL_OFF_gc);

}